<?php

namespace Drupal\blazy\Media;

/**
 * Provides OEmbed integration.
 */
interface BlazyOEmbedInterface {

  /**
   * Returns the Media oEmbed resource fecther.
   */
  public function getResourceFetcher();

  /**
   * Returns the Media oEmbed url resolver fecthers.
   */
  public function getUrlResolver();

  /**
   * Returns the Media oEmbed url resolver fecthers.
   */
  public function getIframeUrlHelper();

  /**
   * Returns the blazy manager.
   */
  public function blazyManager();

  /**
   * Returns the oEmbed Resource based on the given media input url.
   *
   * @param string $input_url
   *   The video url.
   *
   * @return \Drupal\media\OEmbed\Resource
   *   The oEmbed resource.
   */
  public function getResource($input_url);

  /**
   * Builds media-related settings based on the given media input url.
   *
   * Accepts various sources: BlazyFilter, BlazyViewsFieldFile, BlazyEntity,
   * regular Media/ OEmbed related formatters, and the deprecated VEF.
   * Old VEF/BVEF might have no expected entity to associate with.
   *
   * @param array $build
   *   The array being modified containing: content, settings and image item.
   *   Or just settings content for old deprecated approach.
   * @param object $entity
   *   The Media entity, File entity, ER, FieldItemList, etc., optional
   *   to accommodate old approach (pre 2.10) and UGC.
   *
   * @todo should be at non-static BlazyMedia at 4.x, if too late for 3.x.
   * @todo add a return to avoid potential issues with references at 3.x.
   * @todo make it single param like the rest by 3.x.
   */
  public function build(array &$build, $entity = NULL): void;

  /**
   * Checks the given input URL.
   *
   * @param array $settings
   *   The settings being modified.
   * @param string $input
   *   The input to modify.
   *
   * @return string
   *   The modified input url.
   */
  public function checkInputUrl(array &$settings, $input): ?string;

  /**
   * Returns external image item from resource for BlazyFilter or VEF.
   *
   * The settings fallbacks are preserved for minimal BVEF compat.
   *
   * @param array $settings
   *   The settings being modified.
   */
  public function getThumbnail(array &$settings): ?object;

  /**
   * Converts input URL into embed URL.
   *
   * @param object $blazies
   *   The \Drupal\blazy\BlazySettings object.
   * @param string $input
   *   The input to modify.
   * @param array $autoplay
   *   The optional autoplay.
   *
   * @return string
   *   The media oembed url.
   */
  public function toEmbedUrl($blazies, $input, array $autoplay = []): string;

}
